@@ -2,7 +2,7 @@
 *
 * This file is part of BibleTime's source code, http://www.bibletime.info/.
 *
-* Copyright 1999-2014 by the BibleTime developers.
+* Copyright 1999-2016 by the BibleTime developers.
 * The BibleTime source code is licensed under the GNU General Public License version 2.0.
 *
 **********/
@@ -22,59 +22,65 @@
 #include "frontend/cmdiarea.h"
 #include "frontend/display/cdisplay.h"
 #include "frontend/displaywindow/bttoolbarpopupaction.h"
-#include "frontend/displaywindow/btactioncollection.h"
 #include "frontend/displaywindow/btmodulechooserbar.h"
 #include "frontend/displaywindow/btdisplaysettingsbutton.h"
 #include "frontend/keychooser/ckeychooser.h"
 #include "frontend/keychooser/bthistory.h"
 #include "frontend/searchdialog/csearchdialog.h"
 #include "util/cresmgr.h"
-#include "util/geticon.h"
+
+
+namespace {
+
+inline QWidget * getProfileWindow(QWidget * w) {
+    for (; w; w = w->parentWidget())
+        if (QMdiSubWindow * const sw = qobject_cast<QMdiSubWindow *>(w))
+            return sw;
+    return nullptr;
+}
+
+}
 
 
 CDisplayWindow::CDisplayWindow(const QList<CSwordModuleInfo *> & modules, CMDIArea * parent)
         : QMainWindow(parent),
-        m_actionCollection(0),
+        m_actionCollection(nullptr),
         m_mdi(parent),
-        m_keyChooser(0),
-        m_swordKey(0),
+        m_keyChooser(nullptr),
+        m_swordKey(nullptr),
         m_isReady(false),
-        m_moduleChooserBar(0),
-        m_mainToolBar(0),
-        m_buttonsToolBar(0),
-        m_formatToolBar(0),
-        m_headerBar(0),
-        m_popupMenu(0),
-        m_displayWidget(0),
-        m_history(0) {
+        m_moduleChooserBar(nullptr),
+        m_mainToolBar(nullptr),
+        m_buttonsToolBar(nullptr),
+        m_formatToolBar(nullptr),
+        m_headerBar(nullptr),
+        m_popupMenu(nullptr),
+        m_displayWidget(nullptr),
+        m_history(nullptr) {
     setAttribute(Qt::WA_DeleteOnClose); //we want to destroy this window when it is closed
     m_actionCollection = new BtActionCollection(this);
     setModules(modules);
 
     // Connect this to the backend module list changes
-    connect(CSwordBackend::instance(),
-            SIGNAL(sigSwordSetupChanged(CSwordBackend::SetupChangedReason)),
-            SLOT(reload(CSwordBackend::SetupChangedReason)));
+    BT_CONNECT(CSwordBackend::instance(),
+               SIGNAL(sigSwordSetupChanged(CSwordBackend::SetupChangedReason)),
+               SLOT(reload(CSwordBackend::SetupChangedReason)));
     BibleTime* mainwindow = btMainWindow();
-    connect(mainwindow, SIGNAL(toggledTextWindowHeader(bool)), SLOT(slotShowHeader(bool)) );
-    connect(mainwindow, SIGNAL(toggledTextWindowNavigator(bool)), SLOT(slotShowNavigator(bool)) );
-    connect(mainwindow, SIGNAL(toggledTextWindowToolButtons(bool)), SLOT(slotShowToolButtons(bool)) );
-    connect(mainwindow, SIGNAL(toggledTextWindowModuleChooser(bool)), SLOT(slotShowModuleChooser(bool)) );
-    connect(mainwindow, SIGNAL(toggledTextWindowFormatToolbar(bool)), SLOT(slotShowFormatToolBar(bool)) );
+    BT_CONNECT(mainwindow, SIGNAL(toggledTextWindowHeader(bool)),
+               SLOT(slotShowHeader(bool)));
+    BT_CONNECT(mainwindow, SIGNAL(toggledTextWindowNavigator(bool)),
+               SLOT(slotShowNavigator(bool)));
+    BT_CONNECT(mainwindow, SIGNAL(toggledTextWindowToolButtons(bool)),
+               SLOT(slotShowToolButtons(bool)));
+    BT_CONNECT(mainwindow, SIGNAL(toggledTextWindowModuleChooser(bool)),
+               SLOT(slotShowModuleChooser(bool)));
+    BT_CONNECT(mainwindow, SIGNAL(toggledTextWindowFormatToolbar(bool)),
+               SLOT(slotShowFormatToolBar(bool)));
 }
 
 CDisplayWindow::~CDisplayWindow() {
     delete m_swordKey;
-    m_swordKey = 0;
-}
-
-QWidget * CDisplayWindow::getProfileWindow() const {
-    for (QWidget * w = parentWidget(); w; w = w->parentWidget()) {
-        QMdiSubWindow * sw = qobject_cast<QMdiSubWindow *>(w);
-        if (sw)
-            return sw;
-    }
-    return const_cast<CDisplayWindow *>(this);
+    m_swordKey = nullptr;
 }
 
 BibleTime* CDisplayWindow::btMainWindow() {
@@ -83,14 +89,14 @@ BibleTime* CDisplayWindow::btMainWindow() {
 
 void CDisplayWindow::setToolBarsHidden() {
     // Hide current window toolbars
-    if (mainToolBar())
-        mainToolBar()->setHidden(true);
-    if (buttonsToolBar())
-        buttonsToolBar()->setHidden(true);
-    if (moduleChooserBar())
-        moduleChooserBar()->setHidden(true);
-    if (formatToolBar())
-        formatToolBar()->setHidden(true);
+    if (m_mainToolBar)
+        m_mainToolBar->setHidden(true);
+    if (m_buttonsToolBar)
+        m_buttonsToolBar->setHidden(true);
+    if (m_moduleChooserBar)
+        m_moduleChooserBar->setHidden(true);
+    if (m_formatToolBar)
+        m_formatToolBar->setHidden(true);
 }
 void CDisplayWindow::clearMainWindowToolBars() {
     // Clear main window toolbars, except for works toolbar
@@ -114,17 +120,18 @@ const QString CDisplayWindow::windowCaption() {
 }
 
 /** Returns the used modules as a pointer list */
-const QList<const CSwordModuleInfo*> CDisplayWindow::modules() const {
+const BtConstModuleList CDisplayWindow::modules() const {
     return CSwordBackend::instance()->getConstPointerList(m_modules);
 }
 
 /** Store the settings of this window in the given CProfileWindow object. */
-void CDisplayWindow::storeProfileSettings(const QString & windowGroup) {
+void CDisplayWindow::storeProfileSettings(QString const & windowGroup) const {
     BtConfig & conf = btConfig();
 
     conf.beginGroup(windowGroup);
 
-    QWidget * w = getProfileWindow();
+    QWidget const * const w = getProfileWindow(parentWidget());
+    BT_ASSERT(w);
 
     /**
       \note We don't use saveGeometry/restoreGeometry for MDI subwindows,
@@ -133,7 +140,10 @@ void CDisplayWindow::storeProfileSettings(const QString & windowGroup) {
     */
     const QRect rect(w->x(), w->y(), w->width(), w->height());
     conf.setSessionValue<QRect>("windowRect", rect);
-
+    conf.setSessionValue<bool>("staysOnTop",
+                               w->windowFlags() & Qt::WindowStaysOnTopHint);
+    conf.setSessionValue<bool>("staysOnBottom",
+                               w->windowFlags() & Qt::WindowStaysOnBottomHint);
     conf.setSessionValue("maximized", w->isMaximized());
 
     bool hasFocus = (w == dynamic_cast<CDisplayWindow *>(mdi()->activeSubWindow()));
@@ -141,11 +151,8 @@ void CDisplayWindow::storeProfileSettings(const QString & windowGroup) {
     // conf.setSessionValue("type", static_cast<int>(modules().first()->type()));
 
     // Save current key:
-    if (key()) {
-        CSwordKey * k = key();
-        sword::VerseKey * vk = dynamic_cast<sword::VerseKey*>(k);
-        QString oldLang;
-        if (vk) {
+    if (CSwordKey * const k = key()) {
+        if (sword::VerseKey * const vk = dynamic_cast<sword::VerseKey *>(k)) {
             // Save keys in english only:
             const QString oldLang = QString::fromLatin1(vk->getLocale());
             vk->setLocale("en");
@@ -157,10 +164,10 @@ void CDisplayWindow::storeProfileSettings(const QString & windowGroup) {
     }
 
     // Save list of modules:
-    QStringList mods;
-    Q_FOREACH (const CSwordModuleInfo * module, modules())
-        mods.append(module->name());
-    conf.setSessionValue("modules", mods);
+    conf.setSessionValue("modules", m_modules);
+
+    // Default for "not a write window":
+    conf.setSessionValue("writeWindowType", int(0));
 
     conf.endGroup();
 }
@@ -170,7 +177,8 @@ void CDisplayWindow::applyProfileSettings(const QString & windowGroup) {
     conf.beginGroup(windowGroup);
     setUpdatesEnabled(false);
 
-    QWidget * w = getProfileWindow();
+    QWidget * const w = getProfileWindow(parentWidget());
+    BT_ASSERT(w);
 
     /**
       \note We don't use restoreGeometry/saveGeometry for MDI subwindows,
@@ -180,7 +188,10 @@ void CDisplayWindow::applyProfileSettings(const QString & windowGroup) {
     const QRect rect = conf.sessionValue<QRect>("windowRect");
     w->resize(rect.width(), rect.height());
     w->move(rect.x(), rect.y());
-
+    if (conf.sessionValue<bool>("staysOnTop", false))
+        w->setWindowFlags(w->windowFlags() | Qt::WindowStaysOnTopHint);
+    if (conf.sessionValue<bool>("staysOnBottom", false))
+        w->setWindowFlags(w->windowFlags() | Qt::WindowStaysOnBottomHint);
     if (conf.sessionValue<bool>("maximized"))
         w->showMaximized();
 
@@ -205,13 +216,13 @@ void CDisplayWindow::insertKeyboardActions( BtActionCollection* a ) {
     actn->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_L));
     a->addAction("openLocation", actn);
 
-    actn = new QAction(QIcon(util::getIcon(CResMgr::displaywindows::general::search::icon)),
+    actn = new QAction(CResMgr::displaywindows::general::search::icon(),
                        tr("Search with works of this window"), a);
     actn->setShortcut(CResMgr::displaywindows::general::search::accel);
     a->addAction(CResMgr::displaywindows::general::search::actionName, actn);
 
     BtToolBarPopupAction* action = new BtToolBarPopupAction(
-        QIcon(util::getIcon(CResMgr::displaywindows::general::backInHistory::icon)),
+        CResMgr::displaywindows::general::backInHistory::icon(),
         tr("Back in history"),
         a
     );
@@ -219,7 +230,7 @@ void CDisplayWindow::insertKeyboardActions( BtActionCollection* a ) {
     a->addAction(CResMgr::displaywindows::general::backInHistory::actionName, action);
 
     action = new BtToolBarPopupAction(
-        QIcon(util::getIcon(CResMgr::displaywindows::general::forwardInHistory::icon)),
+        CResMgr::displaywindows::general::forwardInHistory::icon(),
         tr("Forward in history"),
         a
     );
@@ -232,62 +243,39 @@ void CDisplayWindow::initActions() {
 
     CDisplayWindow::insertKeyboardActions(ac);
 
-    QAction* actn = ac->action(CResMgr::displaywindows::general::search::actionName);
-    Q_ASSERT(actn != 0);
-    QObject::connect(actn, SIGNAL(triggered()),
-                     this, SLOT(slotSearchInModules()));
-
-    CDisplayConnections* conn = displayWidget()->connectionsProxy();
-
-    actn = ac->action("openLocation");
-    Q_ASSERT(actn != 0);
-    QObject::connect(actn, SIGNAL(triggered()),
-                     this, SLOT(setFocusKeyChooser()));
-    addAction(actn);
-
-    actn = ac->action("selectAll");
-    Q_ASSERT(actn != 0);
-    QObject::connect(actn, SIGNAL(triggered()),
-                     conn, SLOT(selectAll()));
-    addAction(actn);
-
-    actn = ac->action("copySelectedText");
-    Q_ASSERT(actn != 0);
-    QObject::connect(actn, SIGNAL(triggered()),
-                     conn, SLOT(copySelection()));
-    addAction(actn);
-
-    actn = ac->action("findText");
-    Q_ASSERT(actn != 0);
-    QObject::connect(actn, SIGNAL(triggered()),
-                     conn, SLOT(openFindTextDialog()));
-    addAction(actn);
-
-    actn = ac->action(CResMgr::displaywindows::general::backInHistory::actionName);
-    Q_ASSERT(actn != 0);
-    bool ok = QObject::connect(actn,                    SIGNAL(triggered()),
-                               keyChooser()->history(), SLOT(back()));
-    Q_ASSERT(ok);
-    addAction(actn);
-
-    actn = ac->action(CResMgr::displaywindows::general::forwardInHistory::actionName);
-    Q_ASSERT(actn != 0);
-    ok = QObject::connect(actn,                    SIGNAL(triggered()),
-                          keyChooser()->history(), SLOT(fw()));
-    Q_ASSERT(ok);
-    addAction(actn);
+    namespace DWG = CResMgr::displaywindows::general;
+    initAction(DWG::search::actionName,
+               this,
+               &CDisplayWindow::slotSearchInModules);
+    initAddAction("openLocation", this, &CDisplayWindow::setFocusKeyChooser);
+    CDisplayConnections * const conn = displayWidget()->connectionsProxy();
+    initAddAction("selectAll",
+                  conn,
+                  &CDisplayConnections::selectAll);
+    initAddAction("copySelectedText",
+                  conn,
+                  &CDisplayConnections::copySelection);
+    initAddAction("findText",
+                  conn,
+                  &CDisplayConnections::openFindTextDialog);
+    initAddAction(DWG::backInHistory::actionName,
+                  keyChooser()->history(),
+                  &BTHistory::back);
+    initAddAction(DWG::forwardInHistory::actionName,
+                  keyChooser()->history(),
+                  &BTHistory::fw);
 
     ac->readShortcuts("Displaywindow shortcuts");
 }
 
 /** Refresh the settings of this window. */
 void CDisplayWindow::reload(CSwordBackend::SetupChangedReason) {
-    //first make sure all used Sword modules are still present
-    QMutableStringListIterator it(m_modules);
-    while (it.hasNext()) {
-        if (!CSwordBackend::instance()->findModuleByName(it.next())) {
-            it.remove();
-        }
+    { // First make sure all used Sword modules are still present:
+        CSwordBackend & backend = *(CSwordBackend::instance());
+        QMutableStringListIterator it(m_modules);
+        while (it.hasNext())
+            if (!backend.findModuleByName(it.next()))
+                it.remove();
     }
 
     if (m_modules.isEmpty()) {
@@ -295,12 +283,13 @@ void CDisplayWindow::reload(CSwordBackend::SetupChangedReason) {
         return;
     }
 
-    if (keyChooser()) keyChooser()->setModules( modules(), false );
+    if (CKeyChooser * const kc = keyChooser())
+        kc->setModules(modules(), false);
 
     lookup();
 
-    actionCollection()->readShortcuts("DisplayWindow shortcuts");
-    actionCollection()->readShortcuts("Readwindow shortcuts");
+    m_actionCollection->readShortcuts("DisplayWindow shortcuts");
+    m_actionCollection->readShortcuts("Readwindow shortcuts");
     emit sigModuleListSet(m_modules);
 }
 
@@ -339,11 +328,6 @@ void CDisplayWindow::setFilterOptions(const FilterOptions &filterOptions) {
     emit sigFilterOptionsChanged(m_filterOptions);
 }
 
-/** Set the ready status */
-void CDisplayWindow::setReady(bool ready) {
-    m_isReady = ready;
-}
-
 /** Returns true if the window may be closed. */
 bool CDisplayWindow::queryClose() {
     return true;
@@ -356,12 +340,12 @@ void CDisplayWindow::setKeyChooser( CKeyChooser* ck ) {
 
 /** Sets the new sword key. */
 void CDisplayWindow::setKey( CSwordKey* key ) {
-    Q_ASSERT( key );
+    BT_ASSERT(key);
     m_swordKey = key;
 }
 
 BTHistory* CDisplayWindow::history() {
-    if (m_history == 0)
+    if (m_history == nullptr)
         m_history = new BTHistory(this);
     return m_history;
 }
@@ -408,9 +392,8 @@ void CDisplayWindow::setHeaderBar( QToolBar* header ) {
 void CDisplayWindow::setModules( const QList<CSwordModuleInfo*>& newModules ) {
     m_modules.clear();
 
-    foreach (CSwordModuleInfo* mod, newModules) {
+    Q_FOREACH(CSwordModuleInfo const * const mod, newModules)
         m_modules.append(mod->name());
-    }
 }
 
 /** Initialize the window. Call this method from the outside, because calling this in the constructor is not possible! */
@@ -437,7 +420,7 @@ bool CDisplayWindow::init() {
     emit sigFilterOptionsChanged(m_filterOptions);
     emit sigModulesChanged(modules());
 
-    setReady(true);
+    m_isReady = true;
     return true;
 }
 
@@ -468,23 +451,23 @@ void CDisplayWindow::setFormatToolBar( QToolBar* bar ) {
 
 /** Sets the display settings button. */
 void CDisplayWindow::setDisplaySettingsButton(BtDisplaySettingsButton *button) {
-    connect(this, SIGNAL(sigDisplayOptionsChanged(const DisplayOptions&)),
-        button, SLOT(setDisplayOptions(const DisplayOptions&)));
-    connect(this, SIGNAL(sigFilterOptionsChanged(const FilterOptions&)),
-        button, SLOT(setFilterOptions(const FilterOptions&)));
-    connect(this, SIGNAL(sigModulesChanged(const QList<const CSwordModuleInfo*>&)),
-        button, SLOT(setModules(const QList<const CSwordModuleInfo*>&)));
+    BT_CONNECT(this,   SIGNAL(sigDisplayOptionsChanged(DisplayOptions const &)),
+               button, SLOT(setDisplayOptions(DisplayOptions const &)));
+    BT_CONNECT(this,   SIGNAL(sigFilterOptionsChanged(FilterOptions const &)),
+               button, SLOT(setFilterOptions(FilterOptions const &)));
+    BT_CONNECT(this,   SIGNAL(sigModulesChanged(BtConstModuleList const &)),
+               button, SLOT(setModules(BtConstModuleList const &)));
 
     button->setDisplayOptions(displayOptions(), false);
     button->setFilterOptions(filterOptions(), false);
     button->setModules(modules());
 
-    connect(button, SIGNAL(sigFilterOptionsChanged(const FilterOptions&)),
-            this, SLOT(setFilterOptions(const FilterOptions&)));
-    connect(button, SIGNAL(sigDisplayOptionsChanged(const DisplayOptions&)),
-            this, SLOT(setDisplayOptions(const DisplayOptions&)));
-    connect(button, SIGNAL(sigChanged()),
-            this, SLOT(lookup()));
+    BT_CONNECT(button, SIGNAL(sigFilterOptionsChanged(FilterOptions const &)),
+               this,   SLOT(setFilterOptions(FilterOptions const &)));
+    BT_CONNECT(button, SIGNAL(sigDisplayOptionsChanged(DisplayOptions const &)),
+               this,   SLOT(setDisplayOptions(DisplayOptions const &)));
+    BT_CONNECT(button, SIGNAL(sigChanged()),
+               this,   SLOT(lookup()));
 }
 
 void CDisplayWindow::slotShowHeader(bool show) {
@@ -536,19 +519,16 @@ void CDisplayWindow::lookupModKey( const QString& moduleName, const QString& key
     else {     //given module not displayed in this window
         //if the module is displayed in another display window we assume a wrong drop
         //create a new window for the given module
-        QList<CSwordModuleInfo*> mList;
-        mList.append(m);
-
         BibleTime *mainWindow = btMainWindow();
-        Q_ASSERT(mainWindow != 0);
-        mainWindow->createReadDisplayWindow(mList, keyName);
+        BT_ASSERT(mainWindow);
+        mainWindow->createReadDisplayWindow(m, keyName);
     }
 }
 
 void CDisplayWindow::lookupKey( const QString& keyName ) {
     /* This function is called for example after a bookmark was dropped on this window
     */
-    Q_ASSERT(modules().first());
+    BT_ASSERT(modules().first());
 
     lookupModKey(modules().first()->name(), keyName);
 }
@@ -564,7 +544,8 @@ QMenu* CDisplayWindow::popup() {
     // qWarning("CReadWindow::popup()");
     if (!m_popupMenu) {
         m_popupMenu = new QMenu(this);
-        connect(m_popupMenu, SIGNAL(aboutToShow()), this, SLOT(updatePopupMenu()));
+        BT_CONNECT(m_popupMenu, SIGNAL(aboutToShow()),
+                   this,        SLOT(updatePopupMenu()));
         if (displayWidget()) {
             displayWidget()->installPopup(m_popupMenu);
         }
